/*
	{3,3,5}.c
	
	Generate a Spinner Data file in Rez format for 
	the polytope {3,3,5}. (The 120-cell.)
	
	by James Jennings * September 24, 1994

*/

#include <stdio.h>
#include <math.h>
#include <console.h>

#define N0 120		// number of vertices
#define N1 720		// number of edges

int cnt0 = 0; // number of verts seen
int cnt1 = 0; // number of edges seen

const int edgesPerVert = N1*2/N0;	// = 12 in this case

const double t1 = 1.6180340;  // (sqrt(5)+1)/2
const double t2 = 0.6180340;  // 1/t1
const double edgeLength = 1.2361;	// 2*t2 + a little
const int scale = 2000;

typedef double vert[4];
vert thing[N0];

void PrintVert(vert v) {
	// Print a single vertex in Rez format
	int j;
	
	printf("\t\t{ ");
	for (j=0; j<4; j++)
		printf("%5d, ",(short)(v[j]*scale/2));
	printf("},");
}

void PrintAllVert(void) {
	// Print the Rez description of all of the vertices.
	int i;
	printf("resource 'Dim0' (256) {\n\t%d,\n",scale);
	printf("\t{ /* an array of vertices */\n");
	for (i=0; i<cnt0; i++) {
		PrintVert(thing[i]);
		if ( i%10 == 0 || i==cnt0-1) printf("\t/*%4d */",i);
		printf("\n");
	}
	printf("\t}\n");
}

void SetVert(vert v, double x0, double x1, double x2, double x3) {
// Set the values of the vertex
	v[0] = x0; v[1] = x1; v[2] = x2; v[3] = x3;
	//PrintVert(v);
} 

void AddVert(double x, double y, double z, double t) {
// Add a vertex to the list
	if (cnt0>=N0) {
		fprintf(stderr,"too many vertices\n");
		exit(-1);
	}
	SetVert(thing[cnt0],x,y,z,t);
	cnt0++;
}

void AddSignPermuted(double x, double y, double z, double t) {
// Permute all possible signs of the args
	int i,j,k,l;
	for (i = -1; i<=1; i+=2) {
		if ( i==-1 && x==0 ) continue;
		for (j = -1; j<=1; j+=2) {
			if ( j==-1 && y==0 ) continue;
			for (k = -1; k<=1; k+=2) {
				if ( k==-1 && z==0 ) continue;
				for (l = -1; l<=1; l+=2) {
					if ( l==-1 && t==0 ) continue;
						AddVert(i*x, j*y, k*z, l*t);
				}
			}
		}
	}
}

void AddEvenPermuted(double A, double B, double C, double D) {
// The 12 even permutations of 4 objects, passed to the sign permutation stuff
	if (A==B || B==C || C==D) DebugStr("\pBad permutation args");
	AddSignPermuted(A,B,C,D); // base
	AddSignPermuted(B,C,A,D); // cycle xyz
	AddSignPermuted(C,A,B,D); // cycle xyz
	AddSignPermuted(B,A,D,C); // base: swap (xy)(xt)
	AddSignPermuted(A,D,B,C); // cycle xyz
	AddSignPermuted(D,B,A,C); // cycle xyz
	AddSignPermuted(C,D,A,B); // base: swap (xz)(yt)
	AddSignPermuted(D,A,C,B); // cycle xyz
	AddSignPermuted(A,C,D,B); // cycle xyz
	AddSignPermuted(D,C,B,A); // base: swap (xt)(yz)
	AddSignPermuted(C,B,D,A); // cycle xyz
	AddSignPermuted(B,D,C,A); // cycle xyz
}

double Distance(vert v1, vert v2) {
	int i;
	double temp;
	double result = 0;
	for (i=0; i<4; i++) {
		temp = v1[i] - v2[i];
		result += temp*temp;
	}
	return sqrt(result);
}

void PrintEdge(int a, int b) {
	printf("{%3d,%3d}, ",a,b);
}

void PrintAllEdges(void) {
	// Print a Rez compiler description of the edges.
	// Basically, we find the distance between all pairs
	// of vertices, and whenever the distance is right, we use it.
	int i,j;
	double dist;
	int linkCnt;
	
	printf("};\n\nresource 'Dim1' (256) {\n");
	printf("\t{ /* an array of edges */");
	for (i=0; i<cnt0-1; i++) {
		linkCnt = 0; // count how many links to this vertex there are
		for (j=i+1; j<cnt0; j++) {
			dist = Distance(thing[i],thing[j]);
			if ( dist < edgeLength ) {
				if ( cnt1%6 == 0 ) printf("\n/*%3d*/ ",cnt1);
				if ( ++linkCnt > edgesPerVert ) DebugStr("\ptoo many edges here");
			//	printf("Dist = %f\n",dist);	// for debugging
				PrintEdge(i,j);
				cnt1++;	// count the edges found
				if (cnt1>N1) DebugStr("\ptoo many edges");
			}
		}
	//	break; // remove before the final run
	}
	printf("\n\t}\n");
	printf("};\n\n");
}

void GenerateVertices(void) {
	// first the octahedron points (8)
	AddSignPermuted( 2, 0, 0, 0);
	AddSignPermuted( 0, 2, 0, 0);
	AddSignPermuted( 0, 0, 2, 0);
	AddSignPermuted( 0, 0, 0, 2);
	// second the tessaract points (16)
	AddSignPermuted( 1, 1, 1, 1);
	// lastly the interpolated points (96)
	// Note that these are only *even* permutations
	AddEvenPermuted( t1,  1, t2,  0); // base
}

main(int argc, char **argv)
{
	// Allow us to redirect the output to a file.
	_fcreator = 'KAHL';
	argc = ccommand(&argv);
	
	GenerateVertices();

	printf("/*\n\t{3,3,5}.r (the 120-cell)\n*/\n\n");
	printf("\n\n#include \"polytope.r\"\n\n");
	
	PrintAllVert();
	PrintAllEdges();
	
	printf("/* Found %d vertices. */\n",cnt0);
	printf("/* Found %d edges. */\n",cnt1);
	
	return 0;
}
